<?php
namespace App\Controller;

use App\Templates;
use App\Exception;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

class WebService {

    const baseOrderUrl = "order/v1.0/orders/";
    const MAX_RETRY = 3;
    
    const IFO_TOKEN_LIFE = 600; // ten minutes

    private $util;
    private $connection;
    private $branchService;
    private $integrationService;
    private $productService;
    private $dispatchService;
    private $accessToken;
    
    private $basePath;
    private $ordersPath;        
    private $processedPath;     
    private $errPath;           
    private $networkErrPath;            

    private $mailList;
    private $serverName;
    
    private $configuration;
    
    public function __construct (\Doctrine\DBAL\Connection $connection,
                                 \App\Util\Util $util,
                                 \App\Service\BranchService $branchService,
                                 \App\Service\ConsumerService $consumerService,
                                 \App\Service\IntegrationService $integrationService,
                                 \App\Service\ProductService $productService,
                                 \App\Service\DispatchService $dispatchService) {

        $this->connection         = $connection;
        $this->util               = $util;
        $this->branchService      = $branchService;
        $this->consumerService    = $consumerService;
        $this->integrationService = $integrationService;
        $this->productService     = $productService;
        $this->dispatchService    = $dispatchService;
    }

    public function welcome(Request $request, Response $response) {
        $sql        = "SELECT CDFILIAL FROM FILIAL";
        $result     = $this->connection->fetchAll($sql);

        $versionInfo = $this->getVersionInfo();

        return $response->withJson([
            'version'     => $versionInfo['version'],
            'revision'    => $versionInfo['revision'],
            'date'        => $versionInfo['date'],
            'branchs'     => count($result)
        ]);
    }

    public function runWebService(Request $request, Response $response) {
        try {
            $req = $request->getParsedBody();
            $configsFile = $this->util->getConfigsFile();
            $orderInfo = $req['orderData'];

            $this->setAccessToken($req['authToken']);

            $orderInfo['displayId'] = (string)$orderInfo['displayId'];

            /*if ($req["auth"] != "81743A8E8D41AF82A4807097CDF4BEC2DDDB9E6334BECCFED4D46FC783A8CA2E") 
                throw new \Exception("Auth inválido.", 666);*/

            $this->configuration = $this->integrationService->getConfiguration();

            if(empty($this->configuration)) {
                $response->withStatus(200);
                return 0;
            }

            $ufTimezone      = (isset($this->configuration['IDUFTIMEZONE']) && !empty($this->configuration['IDUFTIMEZONE'])) ? $this->configuration['IDUFTIMEZONE'] : "SP";
            $trackingEnabled = (isset($this->configuration['IDRASTREIAENTREGA']) && ($this->configuration['IDRASTREIAENTREGA'] == "S"));
            $redirectStore   = (isset($this->configuration['IDUTLVNDPDRPALC']) && ($this->configuration['IDUTLVNDPDRPALC'] == "N"));
            //

            date_default_timezone_set($this->util->getCurrentTimezone($ufTimezone));

            if($this->configuration['IDHABINTE'] == 'N') {
                $response->withStatus(200);
                return 0;
            }

            //Suporte a Entrega Facil (via gestor de pedidos)
            if((!empty($orderInfo['salesChannel'])) && ($orderInfo['salesChannel'] == "POS")) {
                return $response->withJson([
                    'success' => true,
                    'message' => '',
                    'result'  => [
                        'orderId' => $orderInfo['id']
                    ]
                ]);
            }

            $this->mailList  = $this->util->getMailList();

            $this->instanceLogPaths();

            $errorMessage = "";
            $this->connection->beginTransaction();
            $this->productService->clearCombinedProducts();
            
            try {                

                if(bccomp(strlen($orderInfo["delivery"]["deliveryAddress"]["streetNumber"]), "11") != -1) {
                    $orderInfo["delivery"]["deliveryAddress"]["streetNumber"] = "0";
                }
                
                $merchantId = isset($orderInfo["merchant"]["id"]) ? $orderInfo["merchant"]["id"] : null;


                $cdFilial = $this->branchService->fetchBranch($merchantId);

                $currentFile = $this->util->logOrder($this->ordersPath, json_encode($orderInfo), $cdFilial);

                $this->integrationService->validateCashierSinc($cdFilial);

                $comanda = $this->processOrder($orderInfo, $merchantId, $cdFilial, $redirectStore);

                $this->integrationService->validateOrder($comanda['CDFILIAL'], $comanda['NRVENDAREST'], $comanda['PIZZAODDFLAVOR']);

                $this->connection->commit();

                //rotina de dispatch ficará em outro módulo
                if($trackingEnabled) {
                    $orderType = !empty($orderInfo['orderType']) ? $orderInfo['orderType'] : "";
                    $this->dispatchService->registerPendingDispatchOrder($comanda['CDFILIAL'], $comanda['NRCOMANDA'], $comanda['NRCOMANDAEXT'], $orderInfo['id'], $orderType);
                }                    
                
                $this->util->logOrder($this->processedPath, json_encode($orderInfo), $cdFilial);
                
                $this->executeWebService(self::baseOrderUrl.$orderInfo["id"]."/confirm", "POST", 1, true);
                
                $ip = $this->util->getRequestIp();
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Pedido #{$orderInfo['displayId']} filial $cdFilial uuid {$orderInfo['id']} integrado. (IP: {$ip})\n");
            } catch (Exception\NetworkException $e) {
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro de rede (NetworkException): {$e->getMessage()}\n");

                $this->connection->rollback();
                    
                if (isset($orderInfo)) {
                    $cancellationReason = json_decode($this->executeWebService(self::baseOrderUrl.$orderInfo["id"]."/cancellationReasons", "GET"), true);
                    if(isset($cancellationReason["error"])){
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . " Não foi possível recuperar a lista de razões do cancelamento do pedido. O pedidos será cancelado automaticamente no iFood.");
                    } else {
                        foreach($cancellationReason as $reason){
                            if($reason['cancelCodeId'] == "501"){
                                $this->executeWebService(self::baseOrderUrl.$orderInfo["id"]."/requestCancellation", "POST", ["cancellationCode"=> "501", "reason" => "NAO FOI POSSIVEL INTEGRAR O PEDIDO: PARAMETRIZACAO DO PDV INCONSISTENTE."]);
                            }
                        }
                    }
                }
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Pedido #{$orderInfo['displayId']} filial $cdFilial uuid {$orderInfo['id']} não foi integrado, verifique a caixa de e-mails.\n");
                
            } catch (Exception\IntegrationException $e) {
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Error: ".$e->getMessage().PHP_EOL);

                $this->connection->rollback();
                
                if(!isset($cdFilial)) {
                    $cdFilial = time();
                }

                // Não cancelar o pedido no iFood em situações de duplicidade
                $this->util->logError($this->errPath, $e->getMessage(), $cdFilial, $orderInfo["displayId"]);
                return $response->withJson([
                    'success' => true,
                    'message' => 'duplicated',
                    'result'  => [
                        'orderId' => $orderInfo['id']
                    ]
                ]);
            } catch (Exception\ParamsException $e) {
                $this->connection->rollback();
                
                if(!isset($cdFilial)) {
                    $cdFilial = time();
                }
                
                $errorMessage = $e->getMessage();
                
                $this->util->logError($this->errPath, $errorMessage, $cdFilial, $orderInfo["displayId"]);

                $cancellationReason = json_decode($this->executeWebService(self::baseOrderUrl.$orderInfo["id"]."/cancellationReasons", "GET"), true);

                if(isset($cancellationReason["error"])){
                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . " Não foi possível recuperar a lista de razões do cancelamento do pedido. O pedidos será cancelado automaticamente no iFood.");
                } else {
                    foreach($cancellationReason as $reason){
                        if($reason['cancelCodeId'] == "501"){
                            $this->executeWebService(self::baseOrderUrl.$orderInfo["id"]."/requestCancellation", "POST", ["cancellationCode"=> "501", "reason" => "NAO FOI POSSIVEL INTEGRAR O PEDIDO: PARAMETRIZACAO DO PDV INCONSISTENTE."]);
                        }
                    }
                }
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Pedido #{$orderInfo['displayId']} filial $cdFilial uuid {$orderInfo['id']} não foi integrado, verifique a caixa de e-mails.\n");
                
            } catch (Exception\MenuException $e) {
                $this->connection->rollback();
                
                if(!isset($cdFilial)) {
                    $cdFilial = time();
                }
                
                $errorMessage = $e->getMessage();
                
                $this->util->logError($this->errPath, $errorMessage, $cdFilial, $orderInfo["displayId"]);

                $cancellationReason = json_decode($this->executeWebService(self::baseOrderUrl.$orderInfo["id"]."/cancellationReasons", "GET"), true);
                if(isset($cancellationReason["error"])){
                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . " Não foi possível recuperar a lista de razões do cancelamento do pedido. O pedidos será cancelado automaticamente no iFood.");
                } else {
                    foreach($cancellationReason as $reason){
                        if($reason['cancelCodeId'] == "505"){
                            $this->executeWebService(self::baseOrderUrl.$orderInfo["id"]."/requestCancellation", "POST", ["cancellationCode"=> "505", "reason" => "NAO FOI POSSIVEL INTEGRAR O PEDIDO: CARDAPIO INCONSISTENTE."]);
                        }
                    }
                }
                
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Pedido #{$orderInfo['displayId']} filial $cdFilial uuid {$orderInfo['id']} não foi integrado, verifique a caixa de e-mails.\n");
                
            } catch (\Exception $e) {
                $this->connection->rollback();

                $cancellationReason = json_decode($this->executeWebService(self::baseOrderUrl.$orderInfo["id"]."/cancellationReasons", "GET"), true);
                if(isset($cancellationReason["error"])){
                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . " Não foi possível recuperar a lista de razões do cancelamento do pedido. O pedidos será cancelado automaticamente no iFood.");
                } else {
                
                    foreach($cancellationReason as $reason){
                        if($reason['cancelCodeId'] == "501"){
                            $this->executeWebService(self::baseOrderUrl.$orderInfo["id"]."/requestCancellation", "POST", ["cancellationCode"=> "501", "reason" => "NAO FOI POSSIVEL INTEGRAR O PEDIDO."]);
                        }
                    }
                }
                
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Pedido #{$orderInfo['displayId']} filial $cdFilial uuid {$orderInfo['id']} não foi integrado, verifique a caixa de e-mails.\n");
                
                $errorMessage = $e->getMessage();

                $this->util->logError($this->errPath, $errorMessage, $cdFilial, $orderInfo["displayId"]);
            }

            if(!empty($errorMessage)) {
                $currentMailList = !empty($this->mailList) ? $this->mailList : [$configsFile['failoverEmail']];
                $orderNumber = (isset($orderInfo["displayId"]) ? ("#$cdFilial-{$orderInfo['displayId']}") : ("Indeterminado-" . date("H-i-s")));
                
                $template = file_get_contents(__DIR__ ."/../../templates/error.html");
                $this->util->sendMail($currentMailList, [], "Integração Teknisa x iFood", $template, [
                    ":INTEGRACAO"    => "iFood",
                    ":NUMERO_PEDIDO" => $orderNumber,
                    ":NOME_CLIENTE"  => $this->getClientName(),
                    ":NOME_SERVIDOR" => $this->serverName,
                    ":ERRO_MSG"      => $e->getMessage(),
                    ":JSON_PEDIDO"   => json_encode($orderInfo, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE),
                    ":GERADO_EM"     => date("d/m/Y H:i:s")
                ], $currentFile);
                
                return $response->withJson([
                    'success' => false,
                    'orderId' => $req['orderData']['id']
                ]);
            }
            
        } catch (Exception\NetworkException $e) {
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro de rede (NetworkException):  {$e->getMessage()}\n");

        } catch (Exception\GeneralErrorException $e) {
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." " . $e->getMessage() . "\n");
            $this->integrationService->disableIntegration();
            $template = file_get_contents(__DIR__ ."/../../templates/general-error.html");
            $this->util->sendMail((!empty($this->mailList) ? $this->mailList : [$configsFile['failoverEmail']]), [], "Integração Teknisa x iFood", $template, [
                ":INTEGRACAO"    => "iFood",
                ":NOME_CLIENTE"  => $this->getClientName(),
                ":NOME_SERVIDOR" => $this->serverName,
                ":ERRO_MSG"      => $e->getMessage(),
                ":GERADO_EM"     => date("d/m/Y H:i:s")
            ], $currentFile);

        } catch (\Exception $e) {
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro desconhecido: {$e->getMessage()}\n");
            throw new \Exception($e->getMessage());
        }

        return $response->withJson([
            'success' => true,
            'message' => '',
            'result'  => [
                'orderId' => $req['orderData']['id']
            ]
        ]);
    }

    public function cancelOrderRequest(Request $request, Response $response) {
        $req = $request->getParsedBody();

        $order = $req['orderData'];

        $this->setAccessToken($req['authToken']);

        $fileName = round(microtime(true)*1000) . "-" . $order["id"] . ".json";

    	$this->util->writeLog("cancellation", $fileName, $order);
        
        $cancellation = $this->integrationService->cancelOrder($order["id"]."/denyCancellation", "GET");
        
        if(empty($cancellation)) {
            $this->util->writeLog("execution", "", "ERRO - o pedido {$order['id']} não está registrado ou já foi cancelado no sistema \n");
            $this->executeWebService(self::baseOrderUrl.$order["id"]."/denyCancellation", "POST");
        } else {
            $this->util->writeLog("execution", "", "SUCESSO - o pedido {$order['id']} FILIAL: {$cancellation['BRANCHID']} NRCOMANDA: {$cancellation['ORDERNUMBER']} foi cancelado\n");
            $this->executeWebService(self::baseOrderUrl.$order["id"]."/acceptCancellation", "POST");
        }
        
        return $response->withJson([
            'success' => true,
            'orderId' => $req['orderData']['id']
        ]);
    }

    public function cancelOrder(Request $request, Response $response) {
        $req = $request->getParsedBody();

        $order = $req['orderData'];

        $this->setAccessToken($req['authToken']);

        $fileName = round(microtime(true)*1000) . "-" . $order["id"] . ".json";

    	$this->util->writeLog("cancellation", $fileName, $order);
        $cancellation = $this->integrationService->cancelOrder($order["id"]);
        if(empty($cancellation)) {
            $this->util->writeLog("execution", "", "ERRO - o pedido {$order['id']} não está registrado ou já foi cancelado no sistema \n");
        } else {
            $this->util->writeLog("execution", "", "SUCESSO - {$order['id']} FILIAL: {$cancellation['BRANCHID']} NRCOMANDA: {$cancellation['ORDERNUMBER']} foi cancelado \n");
        }
        
        return $response->withJson([
            'success' => true,
            'orderId' => $req['orderData']['id']
        ]);
    }

    public function dispatch(Request $request, Response $response) {
        $req = $request->getParsedBody();
        
        $this->setAccessToken($req['authToken']);
        
        $pendingOrdersFile = $this->util->getPendingOrdersFile();
        $this->filterDispatchFile($pendingOrdersFile);
        $this->processPendingDispatchOrders($pendingOrdersFile);
        $this->processPendingDeliveredOrders($pendingOrdersFile);
    }

    public function getMerchants(Request $request, Response $response) {
        return $response->withJson([
            'merchants' => json_decode($this->executeWebService("v1.0/merchants", "GET"), true)
        ]);
    }

    public function clearDispatchFile(Request $request, Response $response) { 
        $pendingDispatchFile = $this->util->getPendingOrdersFile();
        file_put_contents($pendingDispatchFile, json_encode([], JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        $response->withStatus(200);
    }

    public function debugOrder(Request $request, Response $response) {
        try {
        
            $req = $request->getParsedBody();
            $configsFile = $this->util->getConfigsFile();
            $orderInfo = $req;
            
            $orderInfo['displayId'] = (string)$orderInfo['displayId'];
            
            $orderInfo['customer']['name'] = 'TESTE TEKNISA';

            $this->configuration = $this->integrationService->getConfiguration();

            if(empty($this->configuration)) {
                $response->withStatus(200);
                return 0;
            }

            $ufTimezone      = (isset($this->configuration['IDUFTIMEZONE']) && !empty($this->configuration['IDUFTIMEZONE'])) ? $this->configuration['IDUFTIMEZONE'] : "SP";
            $trackingEnabled = (isset($this->configuration['IDRASTREIAENTREGA']) && ($this->configuration['IDRASTREIAENTREGA'] == "S"));
            $redirectStore   = (isset($this->configuration['IDUTLVNDPDRPALC']) && ($this->configuration['IDUTLVNDPDRPALC'] == "N"));
            //

            date_default_timezone_set($this->util->getCurrentTimezone($ufTimezone));

            if($this->configuration['IDHABINTE'] == 'N') {
                $response->withStatus(200);
                return 0;
            }

            //Suporte a Entrega Facil (via gestor de pedidos)
            if((!empty($orderInfo['salesChannel'])) && ($orderInfo['salesChannel'] == "POS")) {
                return $response->withJson([
                    'success' => true,
                    'message' => '',
                    'result'  => [
                        'orderId' => $orderInfo['id']
                    ]
                ]);
            }

            $this->mailList  = $this->util->getMailList();

            $this->instanceLogPaths();

            $errorMessage = "";
            $this->connection->beginTransaction();
            $this->productService->clearCombinedProducts();
            
            try {               
                // if(bccomp(strlen($orderInfo["delivery"]["deliveryAddress"]["streetNumber"]), "11") != -1){
                //     $orderInfo["delivery"]["deliveryAddress"]["streetNumber"] = "0";
                // }
                
                $mockedMerchants = ["9c7b9611-49e7-45aa-a4d2-afda633eaf51", "e65898b9-b1ab-4c07-8e81-4c74faedc664", "3d7531ef-0536-4143-a23a-77b62af22e21",  "388672d4-0902-41f3-bc5c-95a5b6002a1d"];

                $merchantId = in_array($orderInfo['merchant']['id'], $mockedMerchants) ? $orderInfo['merchant']['id'] : null;
                
                if(empty($merchantId)) {
                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . " Error pedido TESTE: merchantId {$merchantId} não catalogado como debug." . PHP_EOL);
                    
                    return $response->withJson([
                        'success' => false,
                        'orderId' => $req['id']
                    ])->withStatus(400);
                }

                $cdFilial = $this->branchService->fetchBranch($merchantId);
                
                $currentFile = $this->util->logOrder($this->ordersPath, json_encode($orderInfo), $cdFilial);
                

                $this->integrationService->validateCashierSinc($cdFilial);

                $comanda = $this->processOrder($orderInfo, $merchantId, $cdFilial, $redirectStore);

                $this->integrationService->validateOrder($comanda['CDFILIAL'], $comanda['NRVENDAREST'], $comanda['PIZZAODDFLAVOR']);

                $this->connection->commit();
                
                $this->util->logOrder($this->processedPath, json_encode($orderInfo), $cdFilial);
                
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Pedido de TESTE #{$orderInfo['displayId']} filial $cdFilial uuid {$orderInfo['id']} integrado.\n");
            } catch (Exception\NetworkException $e) {
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro de rede (NetworkException): {$e->getMessage()}\n");

                $this->connection->rollback();
            } catch (Exception\IntegrationException $e) {
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Error: ".$e->getMessage().PHP_EOL);

                $this->connection->rollback();
                
                if(!isset($cdFilial)) {
                    $cdFilial = time();
                }

                $this->util->logError($this->errPath, $e->getMessage(), $cdFilial, $orderInfo["displayId"]);
                // Não cancelar o pedido no iFood em situações de duplicidade
            } catch (Exception\ParamsException $e) {
                $this->connection->rollback();
                
                if(!isset($cdFilial)) {
                    $cdFilial = time();
                }
                
                $errorMessage = $e->getMessage();
                
                $this->util->logError($this->errPath, $errorMessage, $cdFilial, $orderInfo["displayId"]);
                
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Pedido de TESTE #{$orderInfo['displayId']} filial $cdFilial uuid {$orderInfo['id']} não foi integrado, verifique a caixa de e-mails.\n");
            } catch (Exception\MenuException $e) {
                $this->connection->rollback();
                
                if(!isset($cdFilial)) {
                    $cdFilial = time();
                }
                
                $errorMessage = $e->getMessage();
                
                $this->util->logError($this->errPath, $errorMessage, $cdFilial, $orderInfo["displayId"]);
                
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Pedido de TESTE #{$orderInfo['displayId']} filial $cdFilial uuid {$orderInfo['id']} não foi integrado, verifique a caixa de e-mails.\n");
            } catch (\Exception $e) {
                $this->connection->rollback();
                
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Pedido de TESTE #{$orderInfo['displayId']} filial $cdFilial uuid {$orderInfo['id']} não foi integrado, verifique a caixa de e-mails.\n");
                
                $errorMessage = $e->getMessage();

                $this->util->logError($this->errPath, $errorMessage, $cdFilial, $orderInfo["displayId"]);
            }
            if(!empty($errorMessage)) {
                $currentMailList = !empty($this->mailList) ? $this->mailList : [$configsFile['failoverEmail']];
                $orderNumber = (isset($orderInfo["displayId"]) ? ("#$cdFilial-{$orderInfo['displayId']}") : ("Indeterminado-" . date("H-i-s")));
                
                $template = file_get_contents(__DIR__ ."/../../templates/error.html");
                $this->util->sendMail($currentMailList, [], "Integração Teknisa x iFood (TESTE)", $template, [
                    ":INTEGRACAO"    => "iFood",
                    ":NUMERO_PEDIDO" => $orderNumber,
                    ":NOME_CLIENTE"  => $this->getClientName(),
                    ":NOME_SERVIDOR" => $this->serverName,
                    ":ERRO_MSG"      => $e->getMessage(),
                    ":JSON_PEDIDO"   => json_encode($orderInfo, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE),
                    ":GERADO_EM"     => date("d/m/Y H:i:s")
                ], $currentFile);
                
                return $response->withJson([
                    'success' => false,
                    'orderId' => $req['id']
                ]);
            }
            
        } catch (Exception\NetworkException $e) {
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro de rede (NetworkException):  {$e->getMessage()}\n");
        } catch (Exception\GeneralErrorException $e) {
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." " . $e->getMessage() . "\n");
            $template = file_get_contents(__DIR__ ."/../../templates/general-error.html");
            $this->util->sendMail((!empty($this->mailList) ? $this->mailList : [$configsFile['failoverEmail']]), [], "Integração Teknisa x iFood (TESTE)", $template, [
                ":INTEGRACAO"    => "iFood",
                ":NOME_CLIENTE"  => $this->getClientName(),
                ":NOME_SERVIDOR" => $this->serverName,
                ":ERRO_MSG"      => $e->getMessage(),
                ":GERADO_EM"     => date("d/m/Y H:i:s")
            ], $currentFile);
        } catch (\Exception $e) {
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro desconhecido: {$e->getMessage()}\n");
            throw new \Exception($e->getMessage());
        }

        return $response->withJson([
            'success' => true,
            'orderId' => $req['id']
        ]);
    }

    private function instanceLogPaths()
    {
        $configsFile = $this->util->getConfigsFile();

        $clientFolder = $this->connection->getParams();
        $clientFolder = (isset($clientFolder['dbname'])) ? $clientFolder['dbname'] : "undefined";

        $this->basePath          = $configsFile['logPath'] . "/$clientFolder";
        $this->ordersPath        = $this->basePath . "/orders";
        $this->processedPath     = $this->basePath . "/processed";
        $this->errPath           = $this->basePath . "/error";
        $this->networkErrPath    = $this->basePath . "/networkError";
        $this->serverName        = $configsFile['server'];
    }
    
    private function filterOrdersArray($ordersEvents, &$orders) {
        foreach($ordersEvents as $ordersEvent) {
            foreach($orders as &$order) {
                if ($ordersEvent['correlationId'] === $order['correlationId'])
                    array_splice($orders, array_search($order, $orders), 1);
            }
            if(isset($order))
                unset($order);
        }
    }
    
    private function verifyOrderDup($unfilteredOrder, &$ordersArray) {
        foreach($ordersArray as $order) {
            if($order['correlationId'] === $unfilteredOrder['correlationId']) {
                return 0;
            }
        }
        array_push($ordersArray, $unfilteredOrder);
    }

    private function filterDispatchFile($pendingOrdersFile) {
        $pendingDispatchFile = file_get_contents($pendingOrdersFile);   
        $pendingDispatchData = json_decode($pendingDispatchFile, true);

        if(empty($pendingDispatchData)) {
            return 0;
        }

        $activeList = [];
        foreach ($pendingDispatchData as $key => $currentOrder) {
            
            $diff = (time() - $currentOrder['createdAt']);
            if($diff > 28800) { //8h
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . " O pedido #" . $currentOrder['nrComandaExt'] . " expirou (não foi expedido dentro do tempo limite de 8h)." . PHP_EOL);
                continue;
            }

            $activeList[$key] = $currentOrder;
        }

        file_put_contents($pendingOrdersFile, json_encode($activeList, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
    }

    private function processPendingDispatchOrders($pendingOrdersFile) {
        $pendingDispatchFile = file_get_contents($pendingOrdersFile);   
        $pendingDispatchData = json_decode($pendingDispatchFile, true);

        if(empty($pendingDispatchData))
            return 0;

        $cdFilialArr     = [];
        $nrComandaArr    = [];
        $nrComandaExtArr = [];

        foreach($pendingDispatchData as $pendingDispatchOrder) {
            
            if(!empty($pendingDispatchOrder['dispatchedAt']) || $pendingDispatchOrder['delivered']) {
                continue;
            }
            
            if(!isset($pendingDispatchOrder['cdFilial']) || empty($pendingDispatchOrder['cdFilial'])) {
                continue;                
            }            
            
            if(!isset($pendingDispatchOrder['nrComanda']) || empty($pendingDispatchOrder['nrComanda'])) {
                continue;                
            }            
            
            if(!isset($pendingDispatchOrder['nrComandaExt']) || empty($pendingDispatchOrder['nrComandaExt'])) {
                continue;                
            }            

            array_push($cdFilialArr,     $pendingDispatchOrder['cdFilial']);
            array_push($nrComandaArr,    $pendingDispatchOrder['nrComanda']);
            array_push($nrComandaExtArr, $pendingDispatchOrder['nrComandaExt']);
        }
        
        if(empty($cdFilialArr))
            return 0;

        if(empty($nrComandaArr))
            return 0;

        if(empty($nrComandaExtArr))
            return 0;
        
        $orders = $this->dispatchService->getPendingDispatchOrders($cdFilialArr, $nrComandaArr, $nrComandaExtArr);
        
        if(empty($orders))
            return 0;
        
        foreach($orders as $order) {
            $key = "{$order['CDFILIAL']}|{$order['NRCOMANDA']}|{$order['NRCOMANDAEXT']}";
            
            if($order['IDSTCOMANDA'] == '4') {
                if(isset($pendingDispatchData[$key])) {
                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . " O pedido #" . $pendingDispatchData[$key]['nrComandaExt'] . " foi removido da fila de controle de entregador (motivo: pedido cancelado)." . PHP_EOL);
                    $pendingDispatchData[$key]['canceled'] = true;
                }
            }
            
            if($order['IDSTCOMANDA'] == '5' || $order['IDSTCOMANDA'] == 'X') {
                if(isset($pendingDispatchData[$key])) {
                    $reference = $pendingDispatchData[$key]['reference'];
                    $orderType = $pendingDispatchData[$key]['orderType'];
                    
                    if ($orderType == "DEFAULT" || $orderType == "DELIVERY") {
                        $this->executeWebService(self::baseOrderUrl . $reference . "/dispatch", "POST");
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . " O pedido #" . $pendingDispatchData[$key]['nrComandaExt'] . " saiu para entrega." . PHP_EOL);
                        $pendingDispatchData[$key]['dispatchedAt'] = time();    
                    } else if ($orderType == "TABLE" || $orderType == "TAKEOUT" || $orderType == "INDOOR") { 
                        $this->executeWebService(self::baseOrderUrl . $reference . "/readyToPickup", "POST");
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . " O pedido #" . $pendingDispatchData[$key]['nrComandaExt'] . " será entregue na sua mesa." . PHP_EOL);
                        $pendingDispatchData[$key]['dispatchedAt'] = time();
                    } else {
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . " O pedido #" . $pendingDispatchData[$key]['nrComandaExt'] . " foi removido da fila de entrega." . PHP_EOL);
                        $pendingDispatchData[$key]['canceled'] = true;
                    }
                }
            }
        }
        
        $activeList = [];
        
        foreach ($pendingDispatchData as $key => $currentOrder) {
            if($currentOrder['canceled']) {
                continue;
            }
            $activeList[$key] = $currentOrder;
        }

        file_put_contents($pendingOrdersFile, json_encode($activeList, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
    }

    private function processPendingDeliveredOrders($pendingOrdersFile) {
        $pendingDeliveredFile = file_get_contents($pendingOrdersFile);   
        $pendingDeliveredData = json_decode($pendingDeliveredFile, true);

        if(empty($pendingDeliveredData))
            return 0;

        foreach($pendingDeliveredData as $key => $pendingDeliveredOrder) {
            
            if(empty($pendingDeliveredOrder['dispatchedAt'])) {
                continue;
            }

            $diff = (time() - $pendingDeliveredOrder['dispatchedAt']);
            if($diff >= 3600) {//3600
                $reference = $pendingDeliveredOrder['reference'];
                //$this->executeWebService(self::baseOrderUrl . $reference . "/statuses/delivery", "POST");

                $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . " O pedido #" . $pendingDeliveredOrder['nrComandaExt'] . " foi entregue." . PHP_EOL);

                $pendingDeliveredData[$key]['delivered'] = true;
            }
        }
    
        $newPendingList = [];
        
        foreach($pendingDeliveredData as $key => $pendingDeliveredOrder) {
            if($pendingDeliveredOrder['delivered']) {
                continue;
            }
            $newPendingList[$key] = $pendingDeliveredOrder;
        }

        file_put_contents($pendingOrdersFile, json_encode($newPendingList, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
    }
    
    private function logCancelOrders($orders) {
        foreach($orders as $order) {
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s"). " [Cancelamento iFood] o pedido reference {$order['correlationId']} foi cancelado no iFood, realizando cancelamento no sistema: ");
            $cancellation = $this->cancelOrder($order);
        }
    }
    
    private function logCancellationRequests($orders) {
        foreach($orders as $order) {
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s"). " [Cancelamento requisitado] o pedido reference {$order['correlationId']} teve seu cancelamento requisitado no iFood, realizando cancelamento no sistema:\n");
            $cancellation = $this->cancelOrder($order);
            if(empty($cancellation)){
                $this->executeWebService("v2.0/orders/".$order["correlationId"]."/statuses/consumerCancellationAccepted", "POST");
            }
        }
    }
    
    private function logCancellationFailed($orders) {
        foreach($orders as $order) {
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s"). " [Cancelamento negado] o pedido reference {$order['correlationId']} teve seu cancelamento negado no iFood\n");
        }
    }

    private function executeWebService($function, $method, $params = [], $retry = 1, $verbose = false) {
        $header = [];
        $configsFile = $this->util->getConfigsFile();
        if(!empty($params)) {
            $header = [
                "Content-type: application/json",
            ];   
        }
        if(strstr($function, "/acceptCancellation") !== false || strstr($function, "/denyCancellation") !== false || strstr($function, "/dispatch") !== false || strstr($function, "/readyToPickup") !== false) {
            array_push($header, 'Content-Length: 0');
        }
        
        if (!empty($this->getAccessToken()))
            array_push($header, "Authorization: bearer ".$this->getAccessToken());
            
        $url = $configsFile['apiURL'] . $function;
        
        //$this->util->writeLog("execution", "", $url);
        
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
        
        if(!empty($params)) {
            curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($params));
        }
        
        curl_setopt($ch, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_TIMEOUT, 10);
        curl_setopt($ch, CURLOPT_FAILONERROR, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        
        $verboseError = '';
        $fileToWrite = null;
        if ($verbose) {
            curl_setopt($ch, CURLOPT_VERBOSE, true);
            $filePath = $this->util->getNetworkVerbosePath();
            $fileToWrite =  fopen($filePath, 'w+');
            curl_setopt($ch, CURLOPT_STDERR, $fileToWrite);

        }

        $curlEventTime = date("d/m/Y H:i:s");
        $initTime = microtime(1);
        $response = curl_exec($ch);
        $endTime  = microtime(1);
        $execTime = round(($endTime - $initTime) * 1000);
            
        //$this->util->writeLog("execution", "", " | " . $execTime . " ms\n");

        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        $curlError = curl_error($ch);
        $curlErrorNumber = curl_errno($ch);
        if($verbose && $fileToWrite) {
            fclose($fileToWrite);
        }
        curl_close($ch);
        
        $requestData = [
            'eventTime' => $curlEventTime,
            'header'    => $header,
            'url'       => $url,
            'method'    => $method,
            'params'    => $params,
            'httpCode'  => $httpCode,
            'error'     => $curlError,
            'response'  => $response,
            'execTime'  => $execTime
        ];
        
        //invalid access token
        //verificar rota no webhook para retornar token valido
 
        if($httpCode == 401) {
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Sem autorização: $function relizando retentativa $retry/".self::MAX_RETRY."\n");
            $this->util->writeLog('network', date("H-i-s") . ".txt", $this->formatNetworkError($requestData));

            $curl = curl_init();

            curl_setopt_array($curl, array(
                CURLOPT_URL => $configsFile['webhookURL'] . 'getCurrentAuthToken',
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => '',
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 10,
                CURLOPT_FOLLOWLOCATION => true,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => 'GET',
            ));

            $response = curl_exec($curl);
            $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);

            if (!empty(curl_error($curl)) || (stripos($response, "<html>"))) {
                $fileName = date("H-i-s") . ".txt";
                $this->util->writeLog('network', date("H-i-s") . ".txt", $this->formatNetworkError([
                    'eventTime' => date("d/m/Y H:i:s"),
                    'header'    => $header,
                    'url'       => $url,
                    'method'    => 'GET',
                    'params'    => [],
                    'httpCode'  => $httpCode,
                    'error'     => curl_error($curl),
                    'response'  => $response
                ]));
            } else{
                $newAccessToken = $response['authToken'];

                $this->setAccessToken($newAccessToken);
                if ($retry <= self::MAX_RETRY) {
                    return $this->executeWebService($function, $method, $params, ($retry + 1));
                }
            }

            curl_close($curl);
        }

        if($curlErrorNumber == CURLE_OPERATION_TIMEDOUT && strstr($function, '/confirm')) {
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Timeout: $function relizando retentativa $retry/".self::MAX_RETRY."\n");
            $this->util->writeLog('network_verbose', date("H-i-s") . ".txt", $this->formatNetworkError($requestData));
            if ($retry <= self::MAX_RETRY) {
                return $this->executeWebService($function, $method, $params, ($retry + 1), true);
            }
        }
        
        if ($httpCode == 404) {
            return "[]";
        }
        
        if (!empty($curlError) || (stripos($response, "<html>"))) {

            $fileName = date("H-i-s") . ".txt";
            $this->util->writeLog('network', $fileName, $this->formatNetworkError($requestData));

            throw new Exception\NetworkException("Não foi possível completar a requisição para a API do iFood");
        }

        return $response;
    }
    
    private function login($route, $params = []) {
        $configsFile = $this->util->getConfigsFile();
        $url    = $configsFile['apiURL'] . $route;
        $header = [
            "Content-Type: multipart/form-data",
            "boundary=----WebKitFormBoundary7MA4YWxkTrZu0gW"
        ];

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
    
        curl_setopt($ch, CURLOPT_POSTFIELDS, $params);
        curl_setopt($ch, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_TIMEOUT, 10);
        curl_setopt($ch, CURLOPT_FAILONERROR, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        
        $response = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        
        if (!empty(curl_error($ch)) || (stripos($response, "<html>"))) {
            $fileName = date("H-i-s") . ".txt";
            $this->util->writeLog('network', $fileName, $this->formatNetworkError([
                'eventTime' => date("d/m/Y H:i:s"),
                'header'    => $header,
                'url'       => $url,
                'method'    => "POST",
                'params'    => $params,
                'httpCode'  => $httpCode,
                'error'     => curl_error($ch),
                'response'  => $response
            ]));

            throw new Exception\NetworkException("Não foi possível completar a requisição para a API do iFood");
        }

        curl_close($ch);
        return $response;
    }
    
    private function formatNetworkError($networkData) {
        $eventTime = $networkData['eventTime'];
        $header    = implode($networkData['header'], "\r\n");
        $url       = $networkData['url'];
        $method    = $networkData['method'];
        $params    = is_array($networkData['params']) ? implode($networkData['params'], "\r\n") : $networkData['params'];
        $httpCode  = $networkData['httpCode'];
        $error     = $networkData['error'];
        $response  = $networkData['response'];

        $template = file_get_contents(__DIR__ . '/../../templates/networkError.txt');
        $templateParams = [
            ':EVENT'     => $eventTime,
            ':HEADER'    => $header,
            ':URL'       => $url,
            ':METHOD'    => $method,
            ':PARAMS'    => $params,
            ':HTTP_CODE' => $httpCode,
            ':ERROR'     => $error,
            ':RESPONSE'  => $response            
        ];

        return str_replace(array_keys($templateParams), array_values($templateParams), $template);
    }   
    
    private function setAccessToken($accessToken) {
        $this->accessToken = $accessToken;
    }
    
    private function getAccessToken() {
        return $this->accessToken;
    }
    
    private function processOrder($order, $merchantId, $cdFilial, $redirectStore) {

        $deliveryRules = $this->integrationService->deliveryRules($cdFilial, $merchantId);
        $generalParams = $this->integrationService->generalParameters($cdFilial, $redirectStore, $deliveryRules['merchantConfiguration']);

        $generalParams["CPF"] = "";

        if (isset($order["customer"]["documentNumber"])) {
            $order["customer"]["documentNumber"] = preg_replace("/[^0-9]/", "", $order["customer"]["documentNumber"]);
            $generalParams["CPF"] = $order["customer"]["documentNumber"];
        }
        
        $consumer = $this->consumerService->processConsumer($generalParams, $order);
        
        return $this->integrationService->openOrderBill($generalParams, $order, $consumer, $deliveryRules);
    }

    private function getClientName() {
        global $CLIENT_NAME;
        return $CLIENT_NAME;
    }
    
    private function getVersionInfo() {
        $file = file_get_contents(__DIR__ . "/../../version.json");
        return json_decode($file, true);
    }
}
